@@ -22,11 +22,6 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 
 '''
-
-import sys
-
-VERSION = (0, 7, 8)
-
 from cymysql import converters
 from cymysql.converters import escape_dict, escape_sequence, escape_string
 from cymysql.err import Warning, Error, InterfaceError, DataError, \
@@ -34,13 +29,14 @@ from cymysql.err import Warning, Error, InterfaceError, DataError, \
      NotSupportedError, ProgrammingError, MySQLError
 from cymysql.times import Date, Time, Timestamp, \
     DateFromTicks, TimeFromTicks, TimestampFromTicks
+from cymysql.connections import Connection
+from cymysql.constants import FIELD_TYPE
 
+VERSION = (0, 9, 16)
 threadsafety = 1
 apilevel = "2.0"
 paramstyle = "format"
 
-from cymysql.connections import Connection
-from cymysql.constants import FIELD_TYPE
 
 class DBAPISet(frozenset):
 
@@ -60,23 +56,26 @@ class DBAPISet(frozenset):
         return frozenset.__hash__(self)
 
 
-STRING    = DBAPISet([FIELD_TYPE.ENUM, FIELD_TYPE.STRING,
-                     FIELD_TYPE.VAR_STRING])
-BINARY    = DBAPISet([FIELD_TYPE.BLOB, FIELD_TYPE.LONG_BLOB,
-                     FIELD_TYPE.MEDIUM_BLOB, FIELD_TYPE.TINY_BLOB])
-NUMBER    = DBAPISet([FIELD_TYPE.DECIMAL, FIELD_TYPE.DOUBLE, FIELD_TYPE.FLOAT,
-                     FIELD_TYPE.INT24, FIELD_TYPE.LONG, FIELD_TYPE.LONGLONG,
-                     FIELD_TYPE.TINY, FIELD_TYPE.YEAR])
-DATE      = DBAPISet([FIELD_TYPE.DATE, FIELD_TYPE.NEWDATE])
-TIME      = DBAPISet([FIELD_TYPE.TIME])
+STRING = DBAPISet([FIELD_TYPE.ENUM, FIELD_TYPE.STRING, FIELD_TYPE.VAR_STRING])
+BINARY = DBAPISet([
+    FIELD_TYPE.BLOB, FIELD_TYPE.LONG_BLOB, FIELD_TYPE.MEDIUM_BLOB, FIELD_TYPE.TINY_BLOB
+])
+NUMBER = DBAPISet([
+    FIELD_TYPE.DECIMAL, FIELD_TYPE.DOUBLE, FIELD_TYPE.FLOAT, FIELD_TYPE.INT24,
+    FIELD_TYPE.LONG, FIELD_TYPE.LONGLONG, FIELD_TYPE.TINY, FIELD_TYPE.YEAR
+])
+DATE = DBAPISet([FIELD_TYPE.DATE, FIELD_TYPE.NEWDATE])
+TIME = DBAPISet([FIELD_TYPE.TIME])
 TIMESTAMP = DBAPISet([FIELD_TYPE.TIMESTAMP, FIELD_TYPE.DATETIME])
-DATETIME  = TIMESTAMP
-ROWID     = DBAPISet()
+DATETIME = TIMESTAMP
+ROWID = DBAPISet()
+
 
 def Binary(x):
     """Return x as a binary type."""
     return bytes(x)
 
+
 def Connect(*args, **kwargs):
     """
     Connect to the database; see connections.Connection.__init__() for
@@ -84,6 +83,7 @@ def Connect(*args, **kwargs):
     """
     return Connection(*args, **kwargs)
 
+
 connect = Connect
 
 NULL = "NULL"
@@ -98,5 +98,5 @@ __all__ = [
     'NotSupportedError', 'DBAPISet', 'OperationalError', 'ProgrammingError',
     'ROWID', 'STRING', 'TIME', 'TIMESTAMP', 'Warning', 'apilevel', 'connect',
     'constants', 'converters', 'cursors', 'escape_dict', 'escape_sequence',
-    'escape_string', 'paramstyle', 'threadsafety', "NULL","__version__",
-    ]
+    'escape_string', 'paramstyle', 'threadsafety', "NULL", "__version__",
+]
