@@ -1,39 +1,44 @@
 # Python implementation of the MySQL client-server protocol
-#   http://forge.mysql.com/wiki/MySQL_Internals_ClientServer_Protocol
+#   https://dev.mysql.com/doc/dev/mysql-server/latest/PAGE_PROTOCOL.html
 
 import hashlib
-sha_new = lambda *args, **kwargs: hashlib.new("sha1", *args, **kwargs)
-
 import socket
 import ssl
 import struct
 import sys
 import os
 import stat
-
+import getpass
 try:
     from ConfigParser import RawConfigParser
 except ImportError:
     from configparser import RawConfigParser
 
-import getpass
-DEFAULT_USER = getpass.getuser()
-
-from cymysql.charset import charset_by_name, charset_by_id
+from cymysql.charset import charset_by_name, encoding_by_charset
 from cymysql.cursors import Cursor
-from cymysql.constants.CLIENT import *
-from cymysql.constants.COMMAND import *
+from cymysql.constants import CLIENT, COMMAND
 from cymysql.converters import decoders, encoders, escape_item
-from cymysql.err import raise_mysql_exception, Warning, Error, \
+from cymysql.err import Warning, Error, \
      InterfaceError, DataError, DatabaseError, OperationalError, \
      IntegrityError, InternalError, NotSupportedError, ProgrammingError
 from cymysql.packet import MysqlPacket, MySQLResult
 
 PYTHON3 = sys.version_info[0] > 2
+if PYTHON3:
+    # suppress flake8 error
+    unicode = str
 
-DEBUG = False
+DEFAULT_USER = getpass.getuser()
+DEFAULT_CHARSET = 'utf8mb4'
+
+
+def sha_new(*args, **kwargs):
+    return hashlib.new("sha1", *args, **kwargs)
+
+
+def sha256_new(*args, **kwargs):
+    return hashlib.new("sha256", *args, **kwargs)
 
-DEFAULT_CHARSET = 'utf8'
 
 def byte2int(b):
     if isinstance(b, int):
@@ -41,60 +46,56 @@ def byte2int(b):
     else:
         return ord(b)
 
+
 def int2byte(i):
     if PYTHON3:
         return bytes([i])
     else:
         return chr(i)
 
+
 def pack_int24(n):
     if PYTHON3:
-        return bytes([n&0xFF, (n>>8)&0xFF,(n>>16)&0xFF])
+        return bytes([n & 0xFF, (n >> 8) & 0xFF, (n >> 16) & 0xFF])
     else:
-        return chr(n&0xFF) + chr((n>>8)&0xFF) + chr((n>>16)&0xFF)
+        return chr(n & 0xFF) + chr((n >> 8) & 0xFF) + chr((n >> 16) & 0xFF)
 
-def dump_packet(data):
-    
-    def is_ascii(data):
-        if byte2int(data) >= 65 and byte2int(data) <= 122: #data.isalnum():
-            return data
-        return '.'
-    print("packet length %d" % len(data))
-    print("method call[1]: %s" % sys._getframe(1).f_code.co_name)
-    print("method call[2]: %s" % sys._getframe(2).f_code.co_name)
-    print("method call[3]: %s" % sys._getframe(3).f_code.co_name)
-    print("method call[4]: %s" % sys._getframe(4).f_code.co_name)
-    print("method call[5]: %s" % sys._getframe(5).f_code.co_name)
-    print("-" * 88)
-    dump_data = [data[i:i+16] for i in range(len(data)) if i%16 == 0]
-    for d in dump_data:
-        print(' '.join(map(lambda x:"%02X" % byte2int(x), d)) + \
-                '   ' * (16 - len(d)) + ' ' * 2 + \
-                ' '.join(map(lambda x:"%s" % is_ascii(x), d)))
-    print("-" * 88)
-    print("")
 
-def _scramble(password, message):
-    if password == None or len(password) == 0:
-        return int2byte(0)
-    if DEBUG: print('password=' + password)
-    stage1 = sha_new(password).digest()
-    stage2 = sha_new(stage1).digest()
-    s = sha_new()
-    s.update(message)
-    s.update(stage2)
-    result = s.digest()
-    return _my_crypt(result, stage1)
+SCRAMBLE_LENGTH = 20
 
-def _my_crypt(message1, message2):
-    length = len(message1)
-    result = struct.pack('B', length)
-    for i in range(length):
-        x = (struct.unpack('B', message1[i:i+1])[0] ^ \
-             struct.unpack('B', message2[i:i+1])[0])
+
+def _xor(data1, data2):
+    result = b''
+    for i in range(len(data1)):
+        j = i % len(data2)
+        x = (struct.unpack('B', data1[i:i+1])[0] ^ struct.unpack('B', data2[j:j+1])[0])
         result += struct.pack('B', x)
     return result
 
+
+def _mysql_native_password_scramble(password, message):
+    if password is None or len(password) == 0:
+        return b''
+    message2 = sha_new(password).digest()
+    stage2 = sha_new(message2).digest()
+    s = sha_new()
+    s.update(message[:SCRAMBLE_LENGTH])
+    s.update(stage2)
+    message1 = s.digest()
+    return _xor(message1, message2)
+
+
+def _caching_sha2_password_scramble(password, nonce):
+    if password is None or len(password) == 0:
+        return b''
+    message1 = sha256_new(password).digest()
+    s = sha256_new()
+    s.update(sha256_new(message1).digest())
+    s.update(nonce[:SCRAMBLE_LENGTH])
+    message2 = s.digest()
+    return _xor(message1, message2)
+
+
 class Connection(object):
     """
     Representation of a socket with a mysql server.
@@ -158,17 +159,17 @@ class Connection(object):
         if compress or named_pipe:
             raise NotImplementedError("compress and named_pipe arguments are not supported")
 
-        if ssl and (ssl.has_key('capath') or ssl.has_key('cipher')):
+        if ssl and ('capath' in ssl or 'cipher' in ssl):
             raise NotImplementedError('ssl options capath and cipher are not supported')
 
-        self.sock = None
+        self.socket = None
         self.ssl = False
         if ssl:
             self.ssl = True
-            client_flag |= SSL
+            client_flag |= CLIENT.SSL
             for k in ('key', 'cert', 'ca'):
                 v = None
-                if ssl.has_key(k):
+                if k in ssl:
                     v = ssl[k]
                 setattr(self, k, v)
 
@@ -190,25 +191,29 @@ class Connection(object):
 
             def _config(key, default):
                 try:
-                    return cfg.get(read_default_group,key)
-                except:
+                    return cfg.get(read_default_group, key)
+                except Exception:
                     return default
 
-            user = _config("user",user)
-            passwd = _config("password",passwd)
+            user = _config("user", user)
+            passwd = _config("password", passwd)
             host = _config("host", host)
-            db = _config("db",db)
-            unix_socket = _config("socket",unix_socket)
+            db = _config("db", db)
+            unix_socket = _config("socket", unix_socket)
             port = _config("port", port)
             charset = _config("default-character-set", charset)
 
-        if (host == 'localhost' and port == 3306
+        if (
+            host == 'localhost' and port == 3306
             and not sys.platform.startswith('win')
-            and (unix_socket is None or not os.path.exists(unix_socket))):
-            for f in ('/var/lib/mysql/mysql.sock',
+            and (unix_socket is None or not os.path.exists(unix_socket))
+        ):
+            for f in (
+                    '/var/lib/mysql/mysql.sock',
                     '/var/run/mysql/mysql.sock',
                     '/var/run/mysql.sock',
-                    '/var/mysql/mysql.sock'):
+                    '/var/mysql/mysql.sock'
+            ):
                 if os.path.exists(f) and stat.S_ISSOCK(os.stat(f).st_mode):
                     unix_socket = f
                     break
@@ -230,10 +235,13 @@ class Connection(object):
         if use_unicode is not None:
             self.use_unicode = use_unicode
 
-        client_flag |= CAPABILITIES
-        client_flag |= MULTI_STATEMENTS
+        self.encoding = encoding_by_charset(self.charset)
+
+        client_flag |= CLIENT.CAPABILITIES
+        client_flag |= CLIENT.MULTI_STATEMENTS
         if self.db:
-            client_flag |= CONNECT_WITH_DB
+            client_flag |= CLIENT.CONNECT_WITH_DB
+        # self.client_flag |= CLIENT.CLIENT_DEPRECATE_EOF
         self.client_flag = client_flag
 
         self.cursorclass = cursorclass
@@ -253,8 +261,6 @@ class Connection(object):
             c = self.cursor()
             c.execute("SET sql_mode=%s", (sql_mode,))
 
-        self.commit()
-
         if init_command is not None:
             c = self.cursor()
             c.execute(init_command)
@@ -265,37 +271,40 @@ class Connection(object):
         ''' Send the quit message and close the socket '''
         if self.socket is None:
             raise Error("Already closed")
-        send_data = struct.pack('<i',1) + int2byte(COM_QUIT)
+        send_data = b'\x01\x00\x00\x00' + int2byte(COMMAND.COM_QUIT)
         self.socket.sendall(send_data)
         self.socket.close()
         self.socket = None
 
     def autocommit(self, value):
         ''' Set whether or not to commit after every execute() '''
+        if value:
+            q = "SET AUTOCOMMIT = 1"
+        else:
+            q = "SET AUTOCOMMIT = 0"
         try:
-            self._execute_command(COM_QUERY, "SET AUTOCOMMIT = %s" % \
-                                      self.escape(value))
+            self._execute_command(COMMAND.COM_QUERY, q)
             self.read_packet()
-        except:
-            exc,value,tb = sys.exc_info()
+        except Exception:
+            exc, value, tb = sys.exc_info()
             self.errorhandler(None, exc, value)
 
     def commit(self):
         ''' Commit changes to stable storage '''
         try:
-            self._execute_command(COM_QUERY, "COMMIT")
+            self._execute_command(COMMAND.COM_QUERY, "COMMIT")
             self.read_packet()
-        except:
-            exc,value,tb = sys.exc_info()
+        except Exception:
+            exc, value, tb = sys.exc_info()
             self.errorhandler(None, exc, value)
 
     def rollback(self):
         ''' Roll back the current transaction '''
         try:
-            self._execute_command(COM_QUERY, "ROLLBACK")
+            self._execute_command(COMMAND.COM_QUERY, "ROLLBACK")
             self.read_packet()
-        except:
-            exc,value,tb = sys.exc_info()
+        except Exception:
+            exc, value, tb = sys.exc_info()
             self.errorhandler(None, exc, value)
 
     def escape(self, obj):
@@ -329,13 +338,11 @@ class Connection(object):
             self.socket = None
 
     def _is_connect(self):
-        return bool(self.sock)
+        return bool(self.socket)
 
     # The following methods are INTERNAL USE ONLY (called from Cursor)
     def query(self, sql):
-        if DEBUG:
-            print("sending query: %s" % sql)
-        self._execute_command(COM_QUERY, sql)
+        self._execute_command(COMMAND.COM_QUERY, sql)
         self._result = MySQLResult(self)
 
     def next_result(self):
@@ -350,24 +357,24 @@ class Connection(object):
     def kill(self, thread_id):
         arg = struct.pack('<I', thread_id)
         try:
-            self._execute_command(COM_PROCESS_KILL, arg)
+            self._execute_command(COMMAND.COM_PROCESS_KILL, arg)
             pkt = self.read_packet()
             return pkt.is_ok_packet()
-        except:
-            exc,value,tb = sys.exc_info()
+        except Exception:
+            exc, value, tb = sys.exc_info()
             self.errorhandler(None, exc, value)
         return False
 
     def ping(self, reconnect=True):
         ''' Check if the server is alive '''
         try:
-            self._execute_command(COM_PING, "")
-        except:
+            self._execute_command(COMMAND.COM_PING, "")
+        except Exception:
             if reconnect:
                 self._connect()
                 return self.ping(False)
             else:
-                exc,value,tb = sys.exc_info()
+                exc, value, tb = sys.exc_info()
                 self.errorhandler(None, exc, value)
                 return
 
@@ -377,42 +384,41 @@ class Connection(object):
     def set_charset(self, charset):
         try:
             if charset:
-                self._execute_command(COM_QUERY, "SET NAMES %s" %
+                self._execute_command(COMMAND.COM_QUERY, "SET NAMES %s" %
                                       self.escape(charset))
                 self.read_packet()
                 self.charset = charset
-        except:
-            exc,value,tb = sys.exc_info()
+        except Exception:
+            exc, value, tb = sys.exc_info()
             self.errorhandler(None, exc, value)
 
     def _connect(self):
+        sock = None
         try:
             if self.unix_socket and (self.host == 'localhost' or self.host == '127.0.0.1'):
                 sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
                 t = sock.gettimeout()
                 sock.settimeout(self.connect_timeout)
                 sock.connect(self.unix_socket)
+                sock.settimeout(t)
                 self.host_info = "Localhost via UNIX socket"
-                if DEBUG: print('connected using unix_socket')
             else:
-                sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-                t = sock.gettimeout()
-                sock.settimeout(self.connect_timeout)
-                sock.connect((self.host, self.port))
+                sock = socket.create_connection((self.host, self.port), self.connect_timeout)
                 self.host_info = "socket %s:%d" % (self.host, self.port)
-                if DEBUG: print('connected using socket')
-            sock.settimeout(t)
         except socket.error as e:
-            sock.close()
-            raise OperationalError(2003, "Can't connect to MySQL server on %r (%s)" % (self.host, e.args[0]))
+            if sock:
+                sock.close()
+            raise OperationalError(
+                2003, "Can't connect to MySQL server on %r (%s)" % (self.host, e.args[0])
+            )
         self.socket = sock
         self._get_server_information()
         self._request_authentication()
 
     def read_packet(self):
-      """Read an entire "mysql packet" in its entirety from the network
-      and return a MysqlPacket type that represents the results."""
-      return MysqlPacket(self)
+        """Read an entire "mysql packet" in its entirety from the network
+        and return a MysqlPacket type that represents the results."""
+        return MysqlPacket(self)
 
     def insert_id(self):
         if self._result:
@@ -420,77 +426,127 @@ class Connection(object):
         else:
             return 0
 
-    def _send_command(self, command, sql):
-        #send_data = struct.pack('<i', len(sql) + 1) + command + sql
-        # could probably be more efficient, at least it's correct
+    def _execute_command(self, command, sql):
         if not self.socket:
             self.errorhandler(None, InterfaceError, (-1, 'socket not found'))
 
-        if ((PYTHON3 and isinstance(sql, str)) or 
-            (not PYTHON3 and  isinstance(sql, unicode))):
-            sql = sql.encode(self.charset)
+        # suppress flake8 error
+        if (
+            (PYTHON3 and isinstance(sql, str)) or
+            (not PYTHON3 and isinstance(sql, unicode))
+        ):
+            sql = sql.encode(self.encoding)
 
+        if len(sql) + 1 > 0xffffff:
+            raise ValueError('Sending query packet is too large')
         prelude = struct.pack('<i', len(sql)+1) + int2byte(command)
         self.socket.sendall(prelude + sql)
-        if DEBUG: dump_packet(prelude + sql)
 
-    def _execute_command(self, command, sql):
-        self._send_command(command, sql)
-        
+    def _scramble(self):
+        if self.auth_plugin_name in ('', 'mysql_native_password'):
+            data = _mysql_native_password_scramble(
+                self.password.encode(self.encoding), self.salt
+            )
+        elif self.auth_plugin_name == 'caching_sha2_password':
+            data = _caching_sha2_password_scramble(
+                self.password.encode(self.encoding), self.salt
+            )
+        elif self.auth_plugin_name == 'mysql_clear_password':
+            data = self.password.encode(self.encoding) + b'\x00'
+        else:
+            raise NotImplementedError(
+                "%s authentication plugin is not implemented" % (self.auth_plugin_name, )
+            )
+        return data
+
     def _request_authentication(self):
-        self._send_authentication()
-
-    def _send_authentication(self):
-        self.client_flag |= CAPABILITIES
-        if self.server_version.startswith('5'):
-            self.client_flag |= MULTI_RESULTS
-
         if self.user is None:
             raise ValueError("Did not specify a username")
 
-        charset_id = charset_by_name(self.charset).id
-        user = self.user.encode(self.charset)
-
-        data_init = struct.pack('<i', self.client_flag) + struct.pack("<I", 1) + \
-                     int2byte(charset_id) + int2byte(0)*23
-
         next_packet = 1
 
-        if self.ssl:
+        charset_id = charset_by_name(self.charset).id
+        user = self.user.encode(self.encoding)
+
+        data_init = (
+            struct.pack('<i', self.client_flag) +
+            struct.pack("<I", 1) +
+            int2byte(charset_id) + int2byte(0)*23
+        )
+
+        if self.ssl and self.server_capabilities & CLIENT.SSL:
             data = pack_int24(len(data_init)) + int2byte(next_packet) + data_init
-            next_packet += 1
-
-            if DEBUG: dump_packet(data)
-
             self.socket.sendall(data)
-            self.socket = ssl.wrap_self.socketet(self.socket, keyfile=self.key,
-                                                 certfile=self.cert,
-                                                 ssl_version=ssl.PROTOCOL_TLSv1,
-                                                 cert_reqs=ssl.CERT_REQUIRED,
-                                                 ca_certs=self.ca)
+            next_packet += 1
+            self.socket = ssl.wrap_socket(self.socket, keyfile=self.key,
+                                          certfile=self.cert,
+                                          ca_certs=self.ca)
 
-        data = data_init + user+int2byte(0) + _scramble(self.password.encode(self.charset), self.salt)
+        data = data_init + user + int2byte(0)
+        authresp = self._scramble()
 
-        if self.db:
-            self.db = self.db.encode(self.charset)
-            data += self.db + int2byte(0)
+        if self.server_capabilities & CLIENT.SECURE_CONNECTION:
+            data += int2byte(len(authresp)) + authresp
+        else:
+            data += authresp + int2byte(0)
+
+        if self.db and self.server_capabilities & CLIENT.CONNECT_WITH_DB:
+            data += self.db.encode(self.encoding) + int2byte(0)
+
+        if self.server_capabilities & CLIENT.PLUGIN_AUTH:
+            data += self.auth_plugin_name.encode(self.encoding) + int2byte(0)
 
         data = pack_int24(len(data)) + int2byte(next_packet) + data
         next_packet += 2
 
-        if DEBUG: dump_packet(data)
-
         self.socket.sendall(data)
-        auth_packet = MysqlPacket(self)
-
-        if DEBUG: dump_packet(auth_packet.get_all_data())
-
-        # if old_passwords is enabled the packet will be 1 byte long and
-        # have the octet 254
+        auth_packet = self.read_packet()
 
         if auth_packet.is_eof_packet():
-            # send legacy handshake
-            raise NotImplementedError("old_passwords are not supported. Check to see if mysqld was started with --old-passwords, if old-passwords=1 in a my.cnf file, or if there are some short hashes in your mysql.user table.")
+            # AuthSwitchRequest
+            # https://dev.mysql.com/doc/internals/en/connection-phase-packets.html#packet-Protocol::AuthSwitchRequest
+            self.auth_plugin_name, self.salt = auth_packet.read_auth_switch_request()
+            data = self._scramble()
+            data = pack_int24(len(data)) + int2byte(next_packet) + data
+            next_packet += 2
+            self.socket.sendall(data)
+            auth_packet = self.read_packet()
+
+        if self.auth_plugin_name == 'caching_sha2_password':
+            self._caching_sha2_authentication2(auth_packet, next_packet)
+
+    def _caching_sha2_authentication2(self, auth_packet, next_packet):
+        # https://dev.mysql.com/doc/dev/mysql-server/latest/page_caching_sha2_authentication_exchanges.html
+        if auth_packet.get_all_data() == b'\x01\x03':   # fast_auth_success
+            self.read_packet()
+            return
+
+        # perform_full_authentication
+        assert auth_packet.get_all_data() == b'\x01\x04'
+
+        if self.ssl or self.unix_socket:
+            data = self.password.encode(self.encoding) + b'\x00'
+        else:
+            # request_public_key
+            data = b'\x02'
+            data = pack_int24(len(data)) + int2byte(next_packet) + data
+            next_packet += 2
+            self.socket.sendall(data)
+            response = self.read_packet()
+            public_pem = response.get_all_data()[1:]
+
+            from Crypto.PublicKey import RSA
+            from Crypto.Cipher import PKCS1_OAEP
+            key = RSA.importKey(public_pem)
+            cipher = PKCS1_OAEP.new(key)
+            password = self.password.encode(self.encoding) + b'\x00'
+            data = cipher.encrypt(_xor(password, self.salt))
+
+        data = pack_int24(len(data)) + int2byte(next_packet) + data
+        next_packet += 2
+        self.socket.sendall(data)
+
+        self.read_packet()
 
     # _mysql support
     def thread_id(self):
@@ -506,40 +562,45 @@ class Connection(object):
         return self.protocol_version
 
     def _get_server_information(self):
+        # https://dev.mysql.com/doc/internals/en/connection-phase-packets.html#packet-Protocol::Handshake
         i = 0
-        packet = MysqlPacket(self)
+        packet = self.read_packet()
         data = packet.get_all_data()
 
-        if DEBUG: dump_packet(data)
-        #packet_len = byte2int(data[i:i+1])
-        #i += 4
         self.protocol_version = byte2int(data[i:i+1])
-
         i += 1
-        server_end = data.find(int2byte(0), i)
-        # TODO: is this the correct charset? should it be default_charset?
-        self.server_version = data[i:server_end].decode(self.charset)
-
-        i = server_end + 1
-        self.server_thread_id = struct.unpack('<h', data[i:i+2])
-
+        str_end = data.find(int2byte(0), i)
+        self.server_version = data[i:str_end].decode('utf-8')
+        i = str_end + 1
+        self.server_thread_id = struct.unpack('<I', data[i:i+4])
         i += 4
         self.salt = data[i:i+8]
-
         i += 9
-        if len(data) >= i + 1:
+        self.server_capabilities = struct.unpack('<H', data[i:i+2])[0]
+        i += 2
+
+        self.server_status = None
+        self.auth_plugin_name = ''
+        if len(data) > i:
+            # Drop server_language and server_charset now.
+            # character_set(1) only the lower 8 bits
+            # self.server_language = byte2int(data[i:i+1])
+            # self.server_charset = charset_by_id(self.server_language).name
+            i += 1
+            self.server_status = struct.unpack('<H', data[i:i+2])[0]
+            i += 2
+            self.server_capabilities |= (struct.unpack('<H', data[i:i+2])[0]) << 16
+            i += 2
+
+            salt_len = byte2int(data[i:i+1])
             i += 1
 
-        self.server_capabilities = struct.unpack('<h', data[i:i+2])[0]
-
-        i += 1
-        self.server_language = byte2int(data[i:i+1])
-        self.server_charset = charset_by_id(self.server_language).name
-
-        i += 16
-        if len(data) >= i+12-1:
-            rest_salt = data[i:i+12]
-            self.salt += rest_salt
+            i += 10     # reserverd
+            if salt_len:
+                rest_salt_len = max(13, salt_len-8)
+                self.salt += data[i:i+rest_salt_len-1]
+                i += rest_salt_len
+            self.auth_plugin_name = data[i:data.find(int2byte(0), i)].decode('utf-8')
 
     def get_server_info(self):
         return self.server_version
